# Class: pe_puppetdb
#
# This class is a wrapper for puppetdb and pe_postgresql modules.
# It provides a simple way to get a PE puppetdb instance up and running
# within Puppet Enterprise.  It will install and configure all necessary packages,
# including the database PostgreSQL server and instance.
#
# In addition to this class, you'll need to configure your puppet master to use
# puppetdb.  You can use the `puppetdb::master::config` class to accomplish this.
#
# Parameters:
#   ['listen_address']     - The address that the web server should bind to
#                            for HTTP requests.  (defaults to `localhost`.
#                            '0.0.0.0' = all)
#   ['listen_port']        - The port on which the puppetdb web server should
#                            accept HTTP requests (defaults to 8080).
#   ['open_listen_port']   - If true, open the http listen port on the firewall. 
#                            (defaults to false).
#   ['ssl_listen_address'] - The address that the web server should bind to
#                            for HTTPS requests.  (defaults to `$::clientcert`.)
#                            Set to '0.0.0.0' to listen on all addresses.
#   ['ssl_listen_port']    - The port on which the puppetdb web server should
#                            accept HTTPS requests (defaults to 8081).
#   ['open_ssl_listen_port'] - If true, open the ssl listen port on the firewall. 
#                            (defaults to true).
#   ['database']           - Which database backend to use; legal values are
#                            `postgres` (default) or `embedded`.  (The `embedded`
#                            db can be used for very small installations or for
#                            testing, but is not recommended for use in production
#                            environments.  For more info, see the puppetdb docs.)
#   ['database_port']      - The port that the database server listens on.
#                            (defaults to `5432`; ignored for `embedded` db)
#   ['database_username']  - The name of the database user to connect as.
#                            (defaults to `puppetdb`; ignored for `embedded` db)
#   ['database_password']  - The password for the database user.
#                            (defaults to `puppetdb`; ignored for `embedded` db)
#   ['database_name']      - The name of the database instance to connect to.
#                            (defaults to `puppetdb`; ignored for `embedded` db)
#   ['tablespace_name']    - The name of puppetdb tablespace.
#   ['tablespace_location']- The location of puppetdb tablespace.
#   ['manage_database']    - Whether to manage the Postgres database. Defaults
#                            to true, can be set to false if the database is
#                            remote and already exists.
#   ['puppetdb_package']   - The puppetdb package name in the package manager
#   ['puppetdb_version']   - The version of the `puppetdb` package that should
#                            be installed.  You may specify an explicit version
#                            number, 'present', or 'latest'.  (defaults to
#                            'present')
#   ['puppetdb_service']   - The name of the puppetdb service.
#   ['confdir']            - The puppetdb configuration directory; defaults to
#                            `/etc/puppetdb/conf.d`.
# Actions:
# - Creates and manages a puppetdb server and its database server/instance.
#
# Requires:
# - `puppetlabs/puppetlabs-pe_postgresql`
#
# Sample Usage:
#   include pe_puppetdb
#
class pe_puppetdb(
  $listen_address            = $puppetdb::params::listen_address,
  $listen_port               = $puppetdb::params::listen_port,
  $open_listen_port          = $puppetdb::params::open_listen_port,
  $ssl_listen_address        = $puppetdb::params::ssl_listen_address,
  $ssl_listen_port           = $puppetdb::params::ssl_listen_port,
  $open_ssl_listen_port      = $puppetdb::params::open_ssl_listen_port,
  $database_host             = $pe_puppetdb::params::database_host,
  $database_port             = $pe_puppetdb::params::database_port,
  $database_username         = $pe_puppetdb::params::database_username,
  $database_password         = $pe_puppetdb::params::database_password,
  $database_name             = $pe_puppetdb::params::database_name,
  $tablespace_name           = $pe_puppetdb::params::tablespace_name,
  $tablespace_location       = $pe_puppetdb::params::tablespace_location,
  $manage_database           = true,
  $puppetdb_package          = $pe_puppetdb::params::puppetdb_package,
  $puppetdb_version          = $puppetdb::params::puppetdb_version,
  $puppetdb_service          = $pe_puppetdb::params::puppetdb_service,
  $confdir                   = $pe_puppetdb::params::confdir,
  $postgres_listen_addresses = undef,
) inherits pe_puppetdb::params {

  $database                  = $pe_puppetdb::params::database

  # Java VM configuration
  $java_args = pe_puppetdb_safe_hiera_hash('pe_puppetdb::java_args', $pe_puppetdb::params::default_jvm_args)

  class { 'puppetdb::server':
    listen_address          => $listen_address,
    listen_port             => $listen_port,
    open_listen_port        => $open_listen_port,
    ssl_listen_address      => $ssl_listen_address,
    ssl_listen_port         => $ssl_listen_port,
    open_ssl_listen_port    => $open_ssl_listen_port,
    database                => $database,
    database_host           => $database_host,
    database_port           => $database_port,
    database_username       => $database_username,
    database_password       => $database_password,
    database_name           => $database_name,
    puppetdb_package        => $puppetdb_package,
    puppetdb_version        => $puppetdb_version,
    puppetdb_service        => $puppetdb_service,
    confdir                 => $confdir,
    java_args               => $java_args,
  }

  if ($manage_database) {
    class { 'pe_puppetdb::database':
      # If this isn't set explicitly, it should be whatever PuppetDB is expecting
      # to connect to postgres on.
      listen_addresses       => $postgres_listen_addresses ? {
        default              => $postgres_listen_addresses,
        undef                => $database_host
      },
      database_name          => $database_name,
      database_username      => $database_username,
      database_password      => $database_password,
      tablespace_name        => $tablespace_name,
      tablespace_location    => $tablespace_location,
      before                 => Class['puppetdb::server'],
    }
  } else {
    # We need the params setup so we can validate the database properly.
    class { 'postgresql::params':
      version                     => '9.2',
      manage_package_repo         => false,
      locale                      => 'en_US.UTF8',
      charset                     => 'UTF8',
      custom_datadir              => "/opt/puppet/var/lib/pgsql/${version}/data",
      custom_confdir              => "/opt/puppet/var/lib/pgsql/${version}/data",
      custom_bindir               => '/opt/puppet/bin',
      custom_client_package_name  => 'pe-postgresql',
      custom_server_package_name  => 'pe-postgresql-server',
      custom_devel_package_name   => 'pe-postgresql-devel',
      custom_java_package_name    => 'pe-postgresql-jdbc',
      custom_service_name         => 'pe-postgresql',
      custom_user                 => 'pe-postgres',
      custom_group                => 'pe-postgres',
      run_initdb                  => false,
    }
  }
}
